package JSTAPd;
use strict;
use warnings;
our $VERSION = '0.01';

1;
__END__

=encoding utf8

=head1 NAME

JSTAPd - testing framework to test Ajax/Javascript libraries

=head1 DESCRIPTION

JSTAPd is a testing framework to test Ajax/Javascript libraries. Test results are generated as TAP test results, so you can use tools like C<prove> to integrate it in your daily testing routines.

=head1 DOCUMENTATIONS

=head2 Introduction

L<JSTAPd::Tutorial::Intro>,

=head2 how to write tests

L<JSTAPd::Tutorial::Test>,

=head2 JavaScript API Reference

L<JSTAPd::Tutorial::Test>,

=head2 For Japanese

L<JSTAPd::Tutorial::JA>,

=head1 AUTHOR

Kazuhiro Osawa E<lt>yappo <at> shibuya <döt> plE<gt>

lestrrat

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
